;(function($, window, undefined) {

    ESHA.config('client', 'recipe editor demo');
    /* uncomment this to use local services
    ESHA.config('api', 'http://localhost:8080');
    ESHA.config('services', '/eshadata/');
    ESHA.recipe.config('api', 'http://localhost:8080');
    ESHA.recipe.config('services', '/eshadata-recipe/');
    //*/

    // internal models
    var data = window.data = store('recipe') || {},
        work = window.work = { track: store('track'), spell: store.ESHA('spell')||true },

    // dom selectors
    dom = {
        form: '.form',
        item: '.item',
        nutrientValue: '.nutrientValue',
        element: 'input,textarea,select,button',
        input: 'input,select',
        file: 'input[type=file]',
        query: '.query',
        measure: '.measure',
        quantity: '.quantity',
        unit: '.unit.available',
        units: '.unit',
        result: '.results option:selected',
        results: '.results',
        resultOptions: '.results option',
        nutrients: 'select[name=nutrients]'
    },

    // direct-access nodes
    body = $('body'),
    main = $('#main');

    // actions (set to path.actions for path.js to handle)
    var actions = path.actions = {
        edit: function() {
            if (!data.text) return path('enter');
            if (data.text == work.lastText) return;
            // clean and repopulate ingredient forms
            $(dom.form).remove();
            work.track = [];
            var opts = { query: data.text, spell: work.spell };
            ESHA.recipe.foods(opts, function(search) {
                for (var i=0,m=search.results.length; i<m; i++) {
                    var parse = search.results[i].parse_info,
                        result = search.results[i].items[0];
                    if (!result) result = {unit:{}};
                    work.track.push({
                        input: parse.query,
                        parseQty: parse.quantity,
                        parseUnit: parse.unit,
                        parseTerms: parse.terms,
                        resultQty: result.quantity,
                        resultUnit: result.unit.id,
                        resultFood: result.id
                    })
                }
                proto('form', search.results);
                store.remove('tracked');
                store('track', work.track);
            });
            work.lastText = data.text;
        },
        analysis: function() {
            if (!(data.items||'').length) return path('edit');
            //TODO: only re-analyze if servings, items or analyzeNutrients have changed
            // clean and repopulate analysis
            $(dom.item+','+dom.nutrientValue).remove();
            proto('item', data.items);
            ESHA.analysis($.extend({}, data), function(analysis) {
                proto('nutrientValue', analysis.results);
            });
        },
        download: function() {
            if (!store('tracked') && !$.browser.msie) {
                for (var i=0,m=data.items.length; i<m; i++) {
                    if (!data.items[i].excluded) {
                        var item = data.items[i],
                            track = work.track[i];
                        track.quantity = item.quantity;
                        track.unit = item.unit.id;
                        track.food = item.id;
                        item.input = track.input;
                        ESHA.recipe.track(track);
                    }
                }
                store('tracked', true);
            }
            var url = ESHA.recipe.save(data, false);
            if (url.length > 2083 && $.browser.msie) {
                //TODO: create/post form instead of displaying json for copy/paste
                return path('save');
            }
            window.location = url;
        },
        save: function() {
            sync.set('json', JSON.stringify(data));
        },
        read: function() {
            var file = $(dom.file)[0].files[0];
            if (!file) return false;
            var name = file.name,
                dot = name.lastIndexOf('.');
            if (dot > 0) name = name.substring(0, dot);
            sync.set('name', name);
            body.addClass('loading');
            reader.readAsText(file);
        },
        load: function() {
            if (!work.json) return false;
            actions.clear(true);
            sync.fill(ESHA.foodlist.enhance(JSON.parse(work.json)));
            var text = '';
            for (var i=0,m=data.items.length; i<m; i++) {
                var item = data.items[i];
                text += item.quantity+' '+item.unit.abbr+' '+item.description +'\n';
            }
            sync.set('text', text);
        },
        clear: function(keepJson) {
            var blank = { text: '', name: '', servings: 1, items: [] };
            if (!keepJson) blank.json = '';
            sync.fill(blank);
        },
        add: function() {
            var values = { parse_info: { query: '' }, items: [] };
            setItems(values.items, proto('form', values), 0);
        },
        search: function() {
            actions.itemSearch(this.parents(dom.form));
        },
        toggle: function() {
            this.parents(dom.form).toggleClass('excluded').trigger('sync');
        },

        // these actions are not called by path
        itemSearch: function(form, more) {
            var query = form.find(dom.query).val(),
                values = form.data('form');
            if (!query) return false;
            var opts = { query: query, spell: work.spell };
            //TODO: revisit this cruft when we move to solr
            var count = values.items.length;
            if (more) {
                opts.count = count < 100 ? count * 2 : count;
                opts.start = 100 + (values.start || 0);
            }
            ESHA.recipe.foods(opts, function(search) {
                values = search.results[0];
                form.data('form', values);
                setItems(values.items, form, values.total);
            });
        },
        selectItem: function(e) {
            var select = $(e.target),
                id = select.val(),
                form = select.parents(dom.form);
            if (id == 'more') {
                actions.itemSearch(form, true);
                return false;
            }
            //TODO: keep active unit value, when possible
            // switch to unit set for selected item
            form.find(dom.unit).removeClass('available');
            form.data('form').units[id].addClass('available');
        },
        enterKey: function(e) {
            if (e.keyCode == 13) {
                var $target = $(e.target);
                if ($target.is(dom.query)) {
                    return actions.itemSearch($target.parents(dom.form));
                }
                var next = $target.next(dom.element);
                if (next) next.focus();
            }
        },
        init: function() {
            actions.init = function(){};//block repeat calls

            // finish dom
            main.removeClass('hidden');
            body.removeClass('loading');
            setOptions($(dom.nutrients), ESHA.nutrients);

            // register model objects with sync
            var n = ESHA.analysis.nParam(ESHA.config('nutrients'));
            sync(['key','nutrients'], {
                key: ESHA.config('key'),
                nutrients: n
            }, true);
            sync(['name','servings','text', 'items*'], data, true);
            sync(['json','spell'], work);
        },
        loading: {
            on: function(){ body.addClass('loading'); },
            off: function(){ body.removeClass('loading'); }
        }
    };
 
    // event listeners
    ESHA(actions.init);// wait for ESHA resources to init
    if ('JSONS' in window) {
        JSONS.on('before', actions.loading.on);
        JSONS.on('after', actions.loading.off);
    }
    $(document)
    .bind('ajaxStart', actions.loading.on)
    .bind('ajaxComplete', actions.loading.off)
    .bind('synced.key', function(e, n, value) {
        if (value !== '') {
            ESHA.config('key', value);
            //ESHA.recipe.config('key', value);
            if (value == ESHA.PUBLIC.KEY) sync.set('key', '');// display blank for public
            else store.ESHA('key', value);
        }
    })
    .bind('synced.spell', function(e, n, value) {
        console.log(e, n, value);
        store.ESHA('spell', value);
    })
    .bind('synced.nutrients', function(e, n, value) {
        ESHA.config('nutrients', value);
        if (!value) sync.set('nutrients', ESHA.nutrient('calories').id);// no null nutrients
        else store.ESHA('nutrients', value);
    })
    .bind('synced', function() {
        store('recipe', data);// overzealously store all changes to recipe data
    })
    .delegate(dom.input, 'keyup', actions.enterKey)
    .delegate(dom.results, 'change', actions.selectItem);
    $(window).bind('error', function(e) {
        alert('There has been an error.\nSee the javascript console for details.');
        console.log('Uncaught error: ', e.originalEvent);
    });

    // custom read/write for form items
    sync.read['items'] = function(node) {
        return {
            excluded: node.is('.excluded'),
            quantity: node.find(dom.quantity).val(),
            unit: ESHA.unit(node.find(dom.unit).val()),
            id: node.find(dom.results).val(),
            description: node.find(dom.result).text()
        };
    };
    var setItems = proto._set.items = function(items, form, total) {
        if (arguments.length == 2) total = items.length;
        form = form.closest(dom.form);
        form.find(dom.resultOptions+','+dom.units).remove();
        var select = form.find(dom.results),
            measure = form.find(dom.measure),
            quantity = measure.find(dom.quantity),
            units = form.data('form').units = {};
        setOptions(select, items);
        form.toggleClass('empty', !items.length);
        for (var i=0; i<items.length; i++) {
            var item = items[i],
                unit = units[item.id] = proto('unit', null, true);
            if (i == 0) {
            	unit.addClass('available');
                quantity.val(item.quantity);
            }
            measure.append(unit);
            setOptions(unit, item.units, item.unit.id);
        }
        if (total > items.length) {
            select.append('<option value="more">More...</option>');
        }
        if (data.items) {
            var item = data.items[form.data('index')];
            if (item) {
                var unit = units[item.id];
                select.val(item.id);
                if (unit) unit.val(item.unit.id || item.unit);
                if (item.excluded) form.addClass('excluded');
                if (item.quantity) quantity.val(item.quantity);
            }
        }
        form.trigger('sync');
    }

    try {// filereader support?
        var reader = new FileReader();
        reader.onload = function(e) {
            sync.set('json', e.target.result);
            body.removeClass('loading');
            //path.js won't wait for file load, so refresh the active panel
            path('load', path.shown[0].id);
        };
    } catch (e) {// nope, use json textarea
        main.addClass('noFileReader');
    }

    // utility function
    function setOptions(select, options, value) {
        select.find('option').remove(); 
        for (var i=0,m=(options||[]).length; i<m; i++) {
            var o = options[i];
            select.append($('<option/>').attr('value', o.id).html(o.abbr || o.description));
        }
        if (value && options) select.val(value);
    }

})(jQuery, window, undefined);