;(function() {

    // main
    var E = window.ESHA = function(fn) {
        if (!E.ready.started) E.loadServices();
        E.ready(fn);
    };

    // constants
    E.VERSION = "1.0";
    E.PUBLIC = {
        KEY: '2s35wsxke74rcut8jqapbkyf',
        API: 'http://api.esha.com',
        TEST_API: 'http://api-test.esha.com',
        SERVICES: '/'
    };

    // utilities
    var toStr = Object.prototype.toString,
        encode = encodeURIComponent;
    E.util = {
        setting: function(key, alt) {
            var val = E.params[key];
            if (val === undefined) val = E.store(key);
            return val === null || val === undefined ? alt : val;
        },
        getParams: function() {
            var vals = {}, href = window.location.href;
            if (href.indexOf('#') > 0) href = href.substring(0, href.indexOf('#'));
            var pairs = href.slice(href.indexOf('?')+1).split('&');
            for (var i=0,m=pairs.length; i<m; i++) {
                var pair = pairs[i].split('='), key = pair[0], val = pair[1];
                if (key in vals) {
                    var was = vals[key];
                    if (E.util.isArray(was)) was.push(val);
                    else vals[key] = [was, val];
                } else {
                    vals[key] = val;
                }
            }
            return vals;
        },
        setParams: function(url, params) {
            var key = E.key(url);
            if (key) params.apikey = key;
            url += '?';
            for (var key in params) {
                url += encode(key)+'='+encode(params[key])+'&';
            }
            return url.substring(0, url.length-1);
        },
        isFunction: function(obj) {
            return toStr.call(obj) === "[object Function]";
        },
        isArray: function(obj) {
            return toStr.call(obj) === "[object Array]";
        },
        round: function(value, decimals) {
            if (!decimals) return Math.round(value);
            var factor = Math.pow(10, decimals);
            return Math.round(factor * value) / factor;
        },
        request: function(opts, source) {
            var key = E.key(opts.url, source);
            if (key) {
                if (!opts.data) opts.data = {};
                opts.data.apikey = key;
            }
            var error = opts.error || function(){ console.log(source+' error', arguments); };
            opts.error = function(xhr, jsons) {
                if (!opts.retries) opts.retries = 0;
                if (opts.retries < config.retries) {
                    opts.retries++;
                    setTimeout(function(){ E.transport(opts); }, config.retryTimeout);
                } else {
                    error.apply(this, arguments);
                }
            };
            var success = opts.success;
            opts.success = function() {
                try {
                    success.apply(this, arguments);
                } catch (e) {
                    console.log(e);
                    opts.retries = config.retries;//prevent retrying
                }
            };
            //TODO: append .js to urls if we need jsonp?
            opts.dataType = 'json';
            E.transport(opts);
        }
    };

    // local resources
    E.store = window.store && store.namespace ? store.namespace('ESHA') : function(){};
    E.params = E.util.getParams();
    E.transport = window.JSONS ? JSONS.get : jQuery.ajax;

    // config
    var config = {
        key: E.util.setting('key') || E.PUBLIC.KEY,
        api: E.util.setting('api', E.PUBLIC.API),
        services: E.util.setting('services', E.PUBLIC.SERVICES),
        retries: E.util.setting('retries', 2),
        retryTimeout: E.util.setting('retryTimeout', 500),
        nutrients: E.util.setting('nutrients', 'calories'),
        info: E.util.setting('info', 55)// max length of food item's info field, terribly named
    };
    E.config = function(key, value) {
        if (typeof key == "string") {
            var dot = key.indexOf('.'), cfg = config;
            if (dot > 0) {
                cfg = cfg[key.substring(0,dot)];
                if (cfg) key = key.substring(dot+1);
                else cfg = config;
            }
            if (value === undefined || value === null || value === '') {
                return cfg[key];
            }
            cfg[key] = value;
        } else {
            for (var k in key) E.config(k, key[k]);
        }
    };

    // remote resources
    E.ready = function(fn) {
        if (E.ready.now && fn !== true) fn(E);
        var queue = E.ready.queue;
        if (fn === true) {
            E.ready.now = true;
            if (queue) while (queue.length) queue.shift()(E);
        }
        else if (queue) queue.push(fn);
        else E.ready.queue = [fn];
    };
    E.load = function(url, fn, source) {
        E.util.request({
            url: url,
            success: function(res) {
                fn(res);
                if (E.isLoaded()) E.ready(true);
            },
            error: function() {
                alert('Could not load: '+url);
            }
        }, source || 'load');
    };
    E.isLoaded = function() {
        return !!(E.nutrients && E.units && E.services);
    };
    E.services = {};
    E.loadServices = function() {
        E.ready.started = true;
        E.load(E.url(config.services), function(services) {
            E.services = services;
            var version = E.store('_version');
            if (version !== services.implementation_version) {
                E.store(false);// clear out the ESHA namespace
                E.store('_version', services.implementation_version);
                E.loadUnits();//which loads nutrients when finished
            } else {
                E.units = E.store('_units');
                E.nutrients = E.store('_nutrients');
            }
        });
    };
    E.loadUnits = function() {
        E.load(E.url('food_units'), function(units) {
            E.units = units;
            var abbr = {
                "Teaspoon": "tsp",      "Tablespoon": "tbsp",   "Cup": "c",
                "Piece": "pc",          "Each": "ea",           "Ounce-weight": "oz",
                "Pound": "lb",          "Gram": "g",            "Kilogram": "kg",
                "Fluid ounce": "fl oz", "Milliliter": "ml",     "Liter": "l",
                "Gallon": "gal",        "Pint": "pt",           "Quart": "qt",
                "Milligram": "mg",      "Microgram": "mcg",     "Serving": "svg"
            };
            for (var i=0,m=units.length; i<m; i++) {
                var u = units[i];
                units[u.description] = units[u.id] = u;//array doubles as hash
                u.lowerCase = u.description.toLowerCase();
                u.abbr = abbr[u.description];
            }
            E.store("_units", units);
            E.loadNutrients();
        });
    };
    E.loadNutrients = function() {
        E.load(E.url('nutrients'), function(nutrients) {
            E.nutrients = nutrients;
            for (var i=0,m=nutrients.length; i<m; i++) {
                var n = nutrients[i];
                    name = n.description = n.description.replace('yramid', 'late');
                nutrients[name] = nutrients[n.id] = n;//array doubles as hash
                n.lowerCase = name.toLowerCase();
                n.index = i;// for sorting
                if (name === 'Dietary Fiber' || name === 'Total Sugars' ||
                    name.indexOf(' Fat') > 0) {
                    n.component = true;// for indenting
                }
                n.decimals = name === 'Calories' ? 0 :
                             name === 'Dietary Fiber' || name === 'Cholesterol' ? 2 :
                             1;// for rounding
            }
            E.store("_nutrients", nutrients);
        });
    };
    E.url = function(service) {
        service = E.services[service] || service;
        return config.api + service;
    };
    E.key = function(url) {
        // when using api[-test].esha.com, an apikey is required
        if (config.api == E.PUBLIC.API || config.api == E.PUBLIC.TEST_API) {
            return config.key;
        }
    };

    // resource services
    E.findResource = function(all, id) {
        if (!all || typeof id !== "string") return;
        id = id.toLowerCase();
        for (var i=0,m=all.length; i<m; i++) {
            var r = all[i];
            if (id === r.id || id === r.lowerCase || id === r.abbr) return r;
        }
        if (id.charAt(id.length) === 's') id = id.substring(0, id.length-1);
        for (var i=0,m=all.length; i<m; i++) {
            var r = all[i];
            if (r.lowerCase.indexOf(id) >= 0) return r;
        }
    };
    E.nutrient = function(id){ return E.findResource(E.nutrients, id); };
    E.unit = function(id){ return E.findResource(E.units, id); };

    // search services
    E.foods = function(query, success) {
        var data = typeof query === "string" ? { query: query } : query,
            opts = E.util.isFunction(success) ? { success: success } : success;
        opts.data = data;
        opts.url = E.url('foods');
        success = opts.success;
        opts.success = function(foods, o) {
            E.foods.enhance(foods, opts);
            success(foods, o);
        };
        E.util.request(opts, 'foods');
    };
    E.foods.enhance = function(foods, opts) {
        foods.input = opts.data.query;
        foods.spellchecked = foods.input !== foods.query;
        foods.count = opts.data.count || foods.items.length;
        foods.start = opts.data.start || 0;
        foods.end = foods.start + foods.items.length - 1;
        if (foods.end + 1 < foods.total) foods.next = foods.end + 1;
        if (foods.start > 0) foods.prev = Math.max(foods.start - foods.count, 0);
        foods.slip = E.foods.slip(foods.total, foods.start, foods.count, opts.slipSize || 14);
        foods.description = E.foods.describe(foods);
        for (var i=0,m=foods.items.length; i<m; i++) {
            E.foods.enhanceItem(foods.items[i]);
        }
    };
    E.foods.enhanceItem = function(food) {
        food.from = food.product || food.supplier || undefined;
        food.title = food.description;
        if (food.from) {
            if (food.from.indexOf('USDA') >= 0) food.from = undefined;// hide USDA
            else food.title += ' ('+food.from+')';
        }
        food.info = food.title;
        if (food.title > config.info) food.title.substring(0, config.info-4) + '...';
        for (var i=0,m=food.units.length; i<m; i++) {
            food.units[i] = E.unit(food.units[i]) || food.units[i];// don't squash unresolved units
        }
        food.unit = E.unit(food.unit) || food.unit;// don't squash unresolved units
    };
    E.foods.filterQuery = function(type, text) {
        var amp = text.indexOf('&'), space = text.indexOf(' ');
        if (amp >= 0) {
            text = text.substring(0, amp) + text.substring(amp+1);
        }
        if (space >= 0) {
            text = '"'+text+'"';
        }
        return type + ':' + text;
    };
    E.foods.slip = function(total, start, count, size) {
        var pages = Math.ceil(total / count);
        if (pages <= 1) return [];
        var current = Math.floor(start / count),
            first = Math.max(0, current - Math.floor(size / 2)),
            last = Math.min(pages, first + size),
            slip = [];
        if (last - first < size) first = Math.max(0, last - size);
        for (var i=first; i < last; i++) {
            slip.push(i * count);
        }
        slip.hasPrev = (first != 0);
        slip.hasNext = (last != pages);
        return slip;
    };
    E.foods.describe = function(foods) {
        var desc = !foods.items.length ? 0 : foods.start + 1;
        if (desc < foods.total) desc += " - " + (foods.next || foods.total);
        return desc+" of "+foods.total+" for '"+foods.query+"'";
    };

    // foodlist verify/save support functions
    E.foodlist = {
        asData: function(list) {
            if (list.id) list = [list];// was single item
            list = E.util.isArray(list) ? { items: list } : list;
            var data = {
                s: list.servings || 1,
                name: list.name || 'unnamed'
            };
            for (var i=0,m=list.items.length; i<m; i++) {
                var item = list.items[i];
                if (!item.excluded) {
                    data['f'+i] = item.id;
                    data['q'+i] = item.quantity;
                    data['u'+i] = item.unit.id || item.unit;
                    if (item.input) data['i'+i] = item.input;
                }
            }
            return data;
        },
        enhance: function(foodlist) {
            for (var i=0,m=foodlist.items.length; i<m; i++) {
                E.foodlist.enhanceItem(foodlist.items[i]);
            }
        },
        enhanceItem: function(item) {
            if (item.quantity.label) item.quantity = item.quantity.label;
            if (!item.unit.id) item.unit = E.unit(item.unit) || item.unit;//don't squash unresolved units
        }
    };

    // analysis services
    E.analysis = function(list, success) {
        var opts = E.util.isFunction(success) ? { success: success } : success;
        opts.data = E.foodlist.asData(list);
        opts.data.n = E.analysis.nParam(list.nutrients || config.nutrients);
        opts.url = E.url('analysis');
        success = opts.success;
        opts.success = function(analysis, o) {
            E.analysis.enhance(analysis);
            success(analysis, o);
        };
        return E.util.request(opts, 'analysis');
    }
    E.analysis.nParam = function(nutrients) {
        var param = '';
        if (typeof nutrients === "string") {
            nutrients = nutrients.indexOf(',') >= 0 ? nutrients.split(',') : [nutrients];
        }
        for (var i=0,m=nutrients.length;i<m;i++) {
            var nutrient = E.nutrient(nutrients[i]) || nutrients[i];
            param += nutrient.id + ',';
        }
        return param.substring(0, param.length-1);
    };
    E.analysis.enhance = function(analysis) {
        for (var i=0,m=analysis.results.length; i<m; i++) {
            E.analysis.enhanceResult(analysis.results[i]);
        }
        analysis.results.sort(E.analysis.sorter);
    };
    E.analysis.enhanceResult = function(result) {
        result.nutrient = E.nutrient(result.nutrient) || result.nutrient;
        result.value = E.util.round(result.value, result.nutrient.decimals);
        // copy units that are also food units up a level for convenience
        result.unit = E.unit(result.nutrient.unit);
    };
    E.analysis.sorter = function(a, b) {
        return a.nutrient.index - b.nutrient.index;
    };

})();