/**
 * Copyright (c) 2011, Nathan Bubna
 * Dual licensed under the MIT and GPL licenses:
 *   http://www.opensource.org/licenses/mit-license.php
 *   http://www.gnu.org/licenses/gpl.html
 *
 * Simple UI framework for Panels, Actions and Their History.
 * Manages panels by showing/hiding elements (DIV usually) with class 'panel',
 * actions by clicks on elements with path attributes (BUTTON usually),
 * and their history via window.location.hash.
 *
 * @version 0.3
 * @name path
 * @author Nathan Bubna
 */
;(function($, window, document, location, history) {
    // main functions
    var path = window.path = function(id) {
        if (path.debug) console.log('path', arguments, this===window?'':this);
        var nodes = $(), hash = '';
        for (var i=0, m=arguments.length, ret, type; i<m && ret !== false; i++) {
            id = $.trim(arguments[i]);
            if (id) type = id.charAt(0).match(/#|@/) ? id.charAt(0) : '';// watch for 'only' type
            if (type) id = id.substring(1);
            if (type != '#') {// if not only panels
                ret = actions[id] ? act.call(this, actions[id], ret) :// a user action?
                      history[id] ? act.call(history, history[id], ret) : id;// a history action?
            }
            if (type != '@' && ret !== false && panels[id]) {// if not only actions, et al
                hash += hash ? '|'+type+id : type+id;
                nodes.push(panels[id][0]);
                if (ret === id) ret = false;//enable 'return path('id');' in actions
            }
        }
        if (nodes.length) {
            if (path.hash != hash) location.hash = path.hash = hash;
            path.shown.hide();
            path.shown = nodes.show();
        }
        return ret;
    };
    path.isShown = function(id) {
        return $.inArray(panels[id] ? panels[id][0] : id, path.shown) >= 0;
    };
    
    // options and plugin hooks
    var opts = path.options = {
        container: document,// button click binding node
        panel: '.panel',// panel selector
        home: '.home',// optional 'home' panel selector
        clickable: '[path]'// selector for clickable elements
    },
    actions = path.actions = {},// container for actions
    panels = path.panels = {},// container for panels
    act = path.act = function(fn, data) {// override to add behavior and/or args to all action fns
        return fn.apply(this, $.isArray(data) ? data : data || []);
    };

    // event handlers
    $(document).ready(function() {
        // update local config vars (in case they were changed)
        actions = path.actions;
        panels = path.panels;
        act = path.act;

        // init panel nodes
        path.home = '';
        path.shown = $();
        $(opts.panel).each(function(i, node) {
            node = $(node).hide();
            var id = node.id = node.attr('id');
            if (node.is(opts.home)) {
                path.home += path.home ? '|'+id : id;
            }
            panels[id] = node;
        });

        // bind events
        $(opts.container).delegate(opts.clickable, 'click', path.click);
        $(window).bind('load hashchange', path.hashchange);
    });
    path.click = function(e) {
        var el = $(e.target);
        path.apply(el, el.attr('path').split('|'));
    };
    path.hashchange = function(e) {
        var hash = location.hash;
        hash = hash && hash.length > 1 ? hash.substring(1) : path.home;
        if (path.hash != hash) path.apply(e, (path.hash = hash).split('|'));
    };

})(jQuery, window, document, window.location, window.history);