/**
 * Copyright (c) 2011, Nathan Bubna
 * Dual licensed under the MIT and GPL licenses:
 *   http://www.opensource.org/licenses/mit-license.php
 *   http://www.gnu.org/licenses/gpl.html
 *
 * Simple UI template support. Mark elements with class 'proto'
 * for easy cloning and data-filling. Requires jQuery. Lacks proper docs.
 *
 * @version 0.2
 * @name proto
 * @author Nathan Bubna
 */
;(function($, window, document) {

    // public access through window.proto
    var proto = window.proto = function(id, values, detached, index) {
        if (proto._debug) console.log('proto(', id, values, detached, index, ')');
        if ($.isArray(values)) {
            var result = $();
            $.each(values, function(i, value) {
                result.push(proto(id, value, detached, i)[0]);
            });
            return result;
        }
        var original = proto[id],
            clone = original.clone(true);
        if (!detached) {
            if (original._next.length) {
                original._next.before(clone);
            } else {
                original._parent.append(clone);
            }
            index = clone.index() || index || 0;
            clone.data('index', index);
        }
        if (values) {
            clone.data(id, values);// save original values in clone data
            proto._fill(clone, values);
        }
        // replace |i in name with |{index}
        if (clone.is('[name]')) {
            var name = clone.attr('name'),
                indexed = name.replace(/\:i/, ':'+index);
            clone.attr('name', indexed);
        }
        // do the same for child nodes
        var named = clone.find('[name]');
        if (named.length) {
            if (!name) {
                name = id+':i';
                indexed = id+':'+index;
            }
            named.filter('[name*="'+name+'"]').each(function(i, node) {
                node = $(node);
                node.attr('name', node.attr('name').replace(name, indexed));
            });
        }
        return clone;
    };

    // configurable setting(s)
    proto._fill = function(node, values, prefix) {
        for (var name in values) {
            var key = (prefix||'')+name,
                value = values[name],
                custom = $.isFunction(proto._set[key]);
            if (!custom && ($.isPlainObject(value) || $.isArray(value))) {
                proto._fill(node, value, key+($.isArray(value) ? ':' : '.'));
            } else {
                var field = node.find('[name*="'+key+'"]');
                if (field.length) {
                    if (custom) {
                        if (proto._debug) console.log('proto._set.'+key, value, field);
                        proto._set[key](value, field);
                    } else {
                        proto._set(key, value, field);
                    }
                }
            }
        }
    };
    proto._set = function(key, value, field) {
        if (proto._debug) console.log('proto._set', key, value, field);
        if (value === undefined || value === null) value = '';
        else if ($.isArray(value) || $.isPlainObject(value)) value = value.toString();
        if (field.size() > 1) {
            field.each(function(){ proto._set(key, value, $(this)); });
        } else if (field.is('input,select,textarea')) {
            field.val(field.is('[multiple]') ? value.split(',') : value);
        } else if (field.is('[value]')) {
            field.attr('value', value);
        } else {
            field.html(value);
        }
    };
    proto._add = function(id, node) {
        if (proto._debug) console.log('proto._add', id, node);
        node._next = node.next();
        node._parent = node.parent();
        proto[id] = node.detach();
    };

    $(document).ready(function() {
        //proto._debug = true;
        // gather nodes
        $('[proto]').each(function(i, node) {
            node = $(node);
            proto._add(node.attr('proto'), node.removeAttr('proto'));
        });
    });
    //proto._debug = true;

})(jQuery, window, document);
