/**
 * Copyright (c) 2011, Nathan Bubna
 * Dual licensed under the MIT and GPL licenses:
 *   http://www.opensource.org/licenses/mit-license.php
 *   http://www.gnu.org/licenses/gpl.html
 */
;(function(window) {
    var _ = {
        version: "2.0",
        areas: {},

        // utilities
        inherit: function(api, o) {
            if (o.__proto__) o.__proto__ = api;
            else for (var p in api) if (!_.hasOwn(o, p)) o[p] = api[p];
            return o;
        },
        stringify: function(d) {
            return d === undefined || typeof d == "function" ? d+'' : JSON.stringify(d);
        },
        parse: function(s) {
            try{ return JSON.parse(s); }catch(e){ return s; }// if it doesn't parse, return as is
        },
        hasOwn: function(o, p){ return Object.prototype.hasOwnProperty.call(o, p); },

        // extension hooks
        fn: function(name, fn){ store[name] = store.session[name] = _.storeAPI[name] = fn; },
        get: function(area, key){ return area.getItem(key); },
        set: function(area, key, string, ow){ area.setItem(key, string); },
        remove: function(area, key){ area.removeItem(key); },

        // core functions
        Store: function(id, area, namespace) {
            var store = _.inherit(_.storeAPI, function(key, data, overwrite) {
                if (arguments.length === 0) return store.getAll();
                if (data !== undefined) return store.set(key, data, overwrite);
                if (typeof key == "string") return store.get(key);
                if (!key) return store.clear();
                return store.setAll(key, data);// overwrite=data, data=key
            });
            store._id = id;
            store._area = area || _.inherit(_.storageAPI, { items: {}, name: 'fake' });
            store._ns = namespace || '';
            if (!_.areas[id]) _.areas[id] = store._area;
            return store;
        },
        storeAPI: {
            // admin functions
            area: function(id, area) {
                var store = this[id];
                if (!store || !store.area) {
                    store = _.Store(id, area, this._ns);//new area-specific api in this namespace
                    if (!this[id]) this[id] = store;
                }
                return store;
            },
            namespace: function(namespace, createSession) {
                if (!namespace) return this._ns ? this._ns.substring(0,this._ns.length-1) : '';
                var ns = namespace, store = this[ns];
                if (!store || !store.namespace) {
                    store = _.Store(this._id, this._area, this._ns+ns+'.');//new namespaced api
                    if (!this[ns]) this[ns] = store;
                    if (createSession) store.area('session', _.areas.session);
                }
                return store;
            },
            isFake: function(){ return this._area.name == 'fake'; },
            toString: function() {
                return 'store'+(this._ns?'.'+this.namespace():'')+'['+this._id+']';
            },

            // storage functions
            has: function(key) {
                if (this._area.has) return this._area.has(this._in(key));//extension hook
                return !!(this._in(key) in this._area);
            },
            size: function(){ return this.keys().length; },
            key: function(i){ return this._out(this._area.key(i)); },
            each: function(fn, end) {
                for (var i=0, m=this._area.length; i<m; i++) {
                    var key = this.key(i);
                    if (key) fn.call(this, key, end);
                    if (m > this._area.length) { m--; i--; }// in case of removeItem
                }
                return end || this;
            },
            keys: function() {
                return this.each(function(k, list){ list.push(k); }, []);
            },
            get: function(key, alt) {
                var s = _.get(this._area, this._in(key));
                return s !== null ? _.parse(s) : alt || s;// support alt for easy default mgmt
            },
            getAll: function() {
                return this.each(function(k, all){ all[k] = this.get(k); }, {});
            },
            set: function(key, data, overwrite) {
                var d = this.get(key);
                if (d != null && overwrite === false) return data;
                return _.set(this._area, this._in(key), _.stringify(data), overwrite) || d;
            },
            setAll: function(data, overwrite) {
                var changed, val;
                for (var key in data) {
                    val = data[key];
                    if (this.set(key, val, overwrite) != val) changed = true;
                }
                return changed;
            },
            remove: function(key) {
                var d = this.get(key);
                _.remove(this._area, this._in(key));
                return d;
            },
            clear: function() {
                if (!this._ns) {
                    this._area.clear();
                    return this;
                }
                return this.each(function(k){ _.remove(this._area, this._in(k)); });
            },
            clearAll: function() {
                var area = this._area;
                for (var id in _.areas) if (_.hasOwn(_.areas, id)) {
                    this._area = _.areas[id];
                    this.clear();
                }
                this._area = area;
                return this;
            },

            // internal use functions
            _in: function(k) {
                if (typeof k != "string") k = _.stringify(k);
                return this._ns ? this._ns + k : k;
            },
            _out: function(k) {
                if (!this._ns) return k;
                if (k.indexOf(this._ns) === 0) return k.substring(this._ns.length);
            }
        },// end _.storeAPI
        storageAPI: {
            length: 0,
            has: function(k){ return _.hasOwn(this.items, k); },
            key: function(i) {
                var c = 0;
                for (var k in this.items) if (this.has(k) && i === c++) return k;
            },
            setItem: function(k, v) {
                if (!this.has(k)) this.length++;
                this.items[k] = v;
            },
            removeItem: function(k) {
                if (this.has(k)) {
                    delete this.items[k];
                    this.length--;
                }
            },
            getItem: function(k){ return this.has(k) ? this.items[k] : null; },
            clear: function(){ for (var k in this.list) this.removeItem(k); },
            toString: function(){ return this.length+' items in '+this.name+'Storage'; }
        }// end _.storageAPI
    }

    // setup the primary store fn
    if (window.store) _.conflict = window.store;
    window.store = store = _.Store('local', window.localStorage);
    store.local = store;// for completeness
    store._ = _;// for extenders and debuggers...
    // safely setup store.session (throws exception in FF for file:/// urls)
    store.area("session", (function(){try{ return sessionStorage; }catch(e){}})());

})(window);
